<?php
/**
 * @package   admintoolswp
 * @copyright Copyright (c)2017-2019 Nicholas K. Dionysopoulos / Akeeba Ltd
 * @license   GNU GPL version 3 or later
 */

namespace Akeeba\AdminTools\Admin\Helper;

use Akeeba\AdminTools\Library\Html\Select as SelectHelper;
use Akeeba\AdminTools\Library\Utils\ArrayHelper;

defined('ADMINTOOLSINC') or die;

class Select
{
	protected static function genericlist($list, $name, $attribs, $selected, $idTag)
	{
		if (empty($attribs))
		{
			$attribs = null;
		}
		else
		{
			$temp = '';
			foreach ($attribs as $key => $value)
			{
				$temp .= $key . ' = "' . $value . '"';
			}
			$attribs = $temp;
		}

		return SelectHelper::genericList($list, $name, $attribs, 'value', 'text', $selected, $idTag);
	}

	public static function valuelist($options, $name, $attribs = null, $selected = null, $ignoreKey = false)
	{
		$list = array();
		foreach ($options as $k => $v)
		{
			if ($ignoreKey)
			{
				$k = $v;
			}
			$list[] = SelectHelper::option($k, $v);
		}

		return self::genericlist($list, $name, $attribs, $selected, $name);
	}

	public static function forcehttps($name, $attribs = null, $selected = null)
	{
		$options = array();

		$options[] = SelectHelper::option('-1', Language::_('COM_ADMINTOOLS_LBL_CONFIGUREWAF_OPT_FORCEHTTPS_IGNORE'));
		$options[] = SelectHelper::option('0', Language::_('COM_ADMINTOOLS_LBL_CONFIGUREWAF_OPT_FORCEHTTPS_NO'));
		$options[] = SelectHelper::option('1', Language::_('COM_ADMINTOOLS_LBL_CONFIGUREWAF_OPT_FORCEHTTPS_YES'));

		return self::genericlist($options, $name, $attribs, $selected, $name);
	}

	public static function booleanlist($name, $attribs = null, $selected = null, $showEmpty = true)
	{
		$options = array();

		if($showEmpty)
		{
			$options[] = SelectHelper::option('-1', '---');
		}

		$options[] = SelectHelper::option('0', Language::_('NO'));
		$options[] = SelectHelper::option('1', Language::_('YES'));

		return self::genericlist($options, $name, $attribs, $selected, $name);
	}

	public static function csrflist($name, $attribs = null, $selected = null)
	{
		$options = array(
			SelectHelper::option('-1', '---'),
			SelectHelper::option('0', Language::_('COM_ADMINTOOLS_LBL_CONFIGUREWAF_OPT_CSRFSHIELD_NO')),
			SelectHelper::option('1', Language::_('COM_ADMINTOOLS_LBL_CONFIGUREWAF_OPT_CSRFSHIELD_BASIC')),
			SelectHelper::option('2', Language::_('COM_ADMINTOOLS_LBL_CONFIGUREWAF_OPT_CSRFSHIELD_ADVANCED'))
		);

		return self::genericlist($options, $name, $attribs, $selected, $name);
	}

	public static function autoroots($name, $attribs = null, $selected = null)
	{
		$options = array(
			SelectHelper::option('-1', '---'),
			SelectHelper::option('0', Language::_('COM_ADMINTOOLS_LBL_HTACCESSMAKER_AUTOROOT_OFF')),
			SelectHelper::option('1', Language::_('COM_ADMINTOOLS_LBL_HTACCESSMAKER_AUTOROOT_STD')),
			SelectHelper::option('2', Language::_('COM_ADMINTOOLS_LBL_HTACCESSMAKER_AUTOROOT_ALT'))
		);

		return self::genericlist($options, $name, $attribs, $selected, $name);
	}

	public static function published($selected = null, $id = 'enabled', $attribs = array())
	{
		$options = array();
		$options[] = SelectHelper::option('', '- ' . Language::_('COM_ADMINTOOLS_LBL_COMMON_SELECTPUBLISHSTATE') . ' -');
		$options[] = SelectHelper::option(0, Language::_('COM_ADMINTOOLS_LBL_COMMON_UNPUBLISHED'));
		$options[] = SelectHelper::option(1, Language::_('COM_ADMINTOOLS_LBL_COMMON_PUBLISHED'));

		return self::genericlist($options, $id, $attribs, $selected, $id);
	}

	/**
	 * Builds the whole list of reasons, it is useful when dealing with forms
	 *
	 * @return array
	 */
	public static function reasons_options()
	{
		$reasons = array(
			'other', 'ipwl', 'ipbl', 'sqlishield', 'antispam',
			'geoblocking', 'rfishield', 'dfishield', 'uploadshield',
			'httpbl', 'loginfailure', 'awayschedule', 'admindir',
			'nonewadmins', 'phpshield',
		);

		$options = array();

		foreach ($reasons as $reason)
		{
			$options[] = SelectHelper::option($reason, Language::_('COM_ADMINTOOLS_LBL_SECURITYEXCEPTION_REASON_' . strtoupper($reason)));
		}

		return $options;
	}

	public static function reasons($selected = null, $id = 'reason', $attribs = array())
	{
		$options = static::reasons_options();

		// Enable miscellaneous reasons, for use in email templates
		if (isset($attribs['misc']))
		{
			$options[] = SelectHelper::option('ipautoban', Language::_('COM_ADMINTOOLS_WAFEMAILTEMPLATE_REASON_IPAUTOBAN'));
			unset($attribs['misc']);
		}

		// Let's sort the list alphabetically
		ArrayHelper::sortObjects($options, 'text');

		if (isset($attribs['all']))
		{
			array_unshift($options, SelectHelper::option('all', Language::_('COM_ADMINTOOLS_LBL_SECURITYEXCEPTION_REASON_ALL')));
			unset($attribs['all']);
		}

		if (!isset($attribs['hideEmpty']))
		{
			array_unshift($options, SelectHelper::option('', '- ' . Language::_('COM_ADMINTOOLS_LBL_SECURITYEXCEPTION_REASON_SELECT') . ' -'));
		}
		else
		{
			unset($attribs['hideEmpty']);
		}

		return self::genericlist($options, $id, $attribs, $selected, $id);
	}

	public static function wwwredirs($name, $attribs = null, $selected = null)
	{
		$options = array(
			SelectHelper::option('-1', '---'),
			SelectHelper::option('0', Language::_('COM_ADMINTOOLS_LBL_HTACCESSMAKER_WWWREDIR_NO')),
			SelectHelper::option('1', Language::_('COM_ADMINTOOLS_LBL_HTACCESSMAKER_WWWREDIR_WWW')),
			SelectHelper::option('2', Language::_('COM_ADMINTOOLS_LBL_HTACCESSMAKER_WWWREDIR_NONWWW'))
		);

		return self::genericlist($options, $name, $attribs, $selected, $name);
	}

	public static function perms($name, $attribs = null, $selected = null)
	{
		$rawperms = array(0400, 0440, 0444, 0600, 0640, 0644, 0660, 0664, 0700, 0740, 0744, 0750, 0754, 0755, 0757, 0770, 0775, 0777);

		$options = array();
		$options[] = SelectHelper::option('', '---');

		foreach ($rawperms as $perm)
		{
			$text = decoct($perm);
			$options[] = SelectHelper::option('0' . $text, $text);
		}

		return self::genericlist($options, $name, $attribs, $selected, $name);
	}

	public static function trsfreqlist($name, $attribs = null, $selected = null)
	{
		$freqs = array('second', 'minute', 'hour', 'day');

		$options = array();
		$options[] = SelectHelper::option('', '---');
		foreach ($freqs as $freq)
		{
			$text = Language::_('COM_ADMINTOOLS_LBL_CONFIGUREWAF_FREQ' . strtoupper($freq));
			$options[] = SelectHelper::option($freq, $text);
		}

		return self::genericlist($options, $name, $attribs, $selected, $name);
	}

	public static function freqlist($name, $attribs = null, $selected = null)
	{
		$freqs = array('hour', 'day', 'month');

		$options = array();
		$options[] = SelectHelper::option('', '---');
		foreach ($freqs as $freq)
		{
			$text = Language::_('COM_ADMINTOOLS_LBL_CONFIGUREWAF_FREQ' . strtoupper($freq));
			$options[] = SelectHelper::option($freq, $text);
		}

		return self::genericlist($options, $name, $attribs, $selected, $name);
	}

	public static function httpschemes($name, $attribs = null, $selected = null)
	{
		$options = array(
			SelectHelper::option('http', Language::_('COM_ADMINTOOLS_LBL_CONFIGUREWAF_IPLOOKUPSCHEME_HTTP')),
			SelectHelper::option('https', Language::_('COM_ADMINTOOLS_LBL_CONFIGUREWAF_IPLOOKUPSCHEME_HTTPS'))
		);

		return self::genericlist($options, $name, $attribs, $selected, $name);
	}

	public static function scanresultstatus($name = '', $attribs = null, $selected = null)
	{
		$options = array(
			SelectHelper::option('new', Language::_('COM_ADMINTOOLS_LBL_SCANALERTS_STATUS_NEW')),
			SelectHelper::option('suspicious', Language::_('COM_ADMINTOOLS_LBL_SCANALERTS_STATUS_SUSPICIOUS')),
			SelectHelper::option('modified', Language::_('COM_ADMINTOOLS_LBL_SCANALERTS_STATUS_MODIFIED')),
		);

		if($name)
		{
			$options = array_unshift($options, SelectHelper::option('', '- ' . Language::_('COM_ADMINTOOLS_LBL_COMMON_SELECTPUBLISHSTATE') . ' -'));

			return self::genericlist($options, $name, $attribs, $selected, $name);
		}
		else
		{
			return $options;
		}
	}

	public static function symlinks($name, $attribs = null, $selected = null)
	{
		$options = array(
			SelectHelper::option('0', Language::_('COM_ADMINTOOLS_LBL_HTACCESSMAKER_SYMLINKS_OFF')),
			SelectHelper::option('1', Language::_('COM_ADMINTOOLS_LBL_HTACCESSMAKER_SYMLINKS_FOLLOW')),
			SelectHelper::option('2', Language::_('COM_ADMINTOOLS_LBL_HTACCESSMAKER_SYMLINKS_IFOWNERMATCH')),
		);

		return self::genericlist($options, $name, $attribs, $selected, $name);
	}

	public static function keepUrlParamsList($name = '', $attribs = null, $selected = null)
	{
		$options = array(
			SelectHelper::option('', '- - -'),
			SelectHelper::option('0', Language::_('COM_ADMINTOOLS_REDIRECTION_KEEPURLPARAMS_LBL_OFF')),
			SelectHelper::option('1', Language::_('COM_ADMINTOOLS_REDIRECTION_KEEPURLPARAMS_LBL_ALL')),
			SelectHelper::option('2', Language::_('COM_ADMINTOOLS_REDIRECTION_KEEPURLPARAMS_LBL_ADD')),
		);

		if($name)
		{
			return self::genericlist($options, $name, $attribs, $selected, $name);
		}
		else
		{
			return $options;
		}
	}

	public static function httpVerbs($name = '', $attribs = null, $selected = null)
	{
		$options = array(
			SelectHelper::option('', '- - -'),
			SelectHelper::option('GET', 'GET'),
			SelectHelper::option('POST', 'POST'),
			SelectHelper::option('PUT', 'PUT'),
			SelectHelper::option('DELETE', 'DELETE'),
			SelectHelper::option('HEAD', 'HEAD'),
			SelectHelper::option('TRACE', 'TRACE'),
		);

		if($name)
		{
			return self::genericlist($options, $name, $attribs, $selected, $name);
		}
		else
		{
			return $options;
		}
	}

	public static function queryParamType($name = '', $attribs = null, $selected = null)
	{
		$options = array(
			SelectHelper::option('', '- - -'),
			SelectHelper::option('E', Language::_('COM_ADMINTOOLS_LBL_WAFBLACKLISTEDREQUEST_QUERY_CONTENT_EXACT')),
			SelectHelper::option('P', Language::_('COM_ADMINTOOLS_LBL_WAFBLACKLISTEDREQUEST_QUERY_CONTENT_PARTIAL')),
			SelectHelper::option('R', Language::_('COM_ADMINTOOLS_LBL_WAFBLACKLISTEDREQUEST_QUERY_CONTENT_REGEX')),
		);

		if($name)
		{
			return self::genericlist($options, $name, $attribs, $selected, $name);
		}
		else
		{
			return $options;
		}
	}

	public static function referrerpolicy($name, $attribs = null, $selected = null)
	{
		$options = array(
			SelectHelper::option('-1', Language::_('COM_ADMINTOOLS_LBL_HTACCESSMAKER_REFERERPOLICY_DISABLED')),
			SelectHelper::option('', Language::_('COM_ADMINTOOLS_LBL_HTACCESSMAKER_REFERERPOLICY_EMPTY')),
			SelectHelper::option('no-referrer', Language::_('COM_ADMINTOOLS_LBL_HTACCESSMAKER_REFERERPOLICY_NOREF')),
			SelectHelper::option('no-referrer-when-downgrade', Language::_('COM_ADMINTOOLS_LBL_HTACCESSMAKER_REFERERPOLICY_NOREF_DOWNGRADE')),
			SelectHelper::option('same-origin', Language::_('COM_ADMINTOOLS_LBL_HTACCESSMAKER_REFERERPOLICY_SAMEORIGIN')),
			SelectHelper::option('origin', Language::_('COM_ADMINTOOLS_LBL_HTACCESSMAKER_REFERERPOLICY_ORIGIN')),
			SelectHelper::option('strict-origin', Language::_('COM_ADMINTOOLS_LBL_HTACCESSMAKER_REFERERPOLICY_STRICTORIGIN')),
			SelectHelper::option('origin-when-cross-origin', Language::_('COM_ADMINTOOLS_LBL_HTACCESSMAKER_REFERERPOLICY_ORIGINCROSS')),
			SelectHelper::option('strict-origin-when-cross-origin', Language::_('COM_ADMINTOOLS_LBL_HTACCESSMAKER_REFERERPOLICY_STRICTORIGINGCROSS')),
			SelectHelper::option('unsafe-url', Language::_('COM_ADMINTOOLS_LBL_HTACCESSMAKER_REFERERPOLICY_UNSAFE')),
		);

		return self::genericlist($options, $name, $attribs, $selected, $name);
	}

	public static function etagtype($name, $attribs = null, $selected = null)
	{
		$options = array(
			SelectHelper::option('default', Language::_('COM_ADMINTOOLS_LBL_HTACCESSMAKER_ETAGTYPE_DEFAULT')),
			SelectHelper::option('full', Language::_('COM_ADMINTOOLS_LBL_HTACCESSMAKER_ETAGTYPE_FULL')),
			SelectHelper::option('sizetime', Language::_('COM_ADMINTOOLS_LBL_HTACCESSMAKER_ETAGTYPE_SIZETIME')),
			SelectHelper::option('size', Language::_('COM_ADMINTOOLS_LBL_HTACCESSMAKER_ETAGTYPE_SIZE')),
			SelectHelper::option('none', Language::_('COM_ADMINTOOLS_LBL_HTACCESSMAKER_ETAGTYPE_NONE')),
		);

		return self::genericlist($options, $name, $attribs, $selected, $name);
	}

	public static function etagtypeIIS($name, $attribs = null, $selected = null)
	{
		$options = array(
			SelectHelper::option('default', Language::_('COM_ADMINTOOLS_LBL_HTACCESSMAKER_ETAGTYPE_DEFAULT')),
			SelectHelper::option('none', Language::_('COM_ADMINTOOLS_LBL_HTACCESSMAKER_ETAGTYPE_NONE')),
		);

		return self::genericlist($options, $name, $attribs, $selected, $name);
	}

	/**
	 * Drop down list of CSV delimiter preference
	 *
	 * @param   string  $name      The field's name
	 * @param   int     $selected  Pre-selected value
	 * @param   array   $attribs   Field attributes
	 *
	 * @return  string  The HTML of the drop-down
	 */
	public static function csvdelimiters($name = 'csvdelimiters', $selected = 1, $attribs = array())
	{
		$options   = array();
		$options[] = SelectHelper::option('1', 'abc, def');
		$options[] = SelectHelper::option('2', 'abc; def');
		$options[] = SelectHelper::option('3', '"abc"; "def"');
		$options[] = SelectHelper::option('-99', Language::_('COM_ADMINTOOLS_IMPORTANDEXPORT_DELIMITERS_CUSTOM'));

		return self::genericlist($options, $name, $attribs, $selected, $name);
	}

	public static function environment($name, $selected, $attribs = array())
	{
		$options   = array();
		$options[] = SelectHelper::option('apache-mod_php', 'Apache + mod_php');
		$options[] = SelectHelper::option('apache-suphp', 'Apache + suPHP');
		$options[] = SelectHelper::option('cgi', 'Apache + CGI/FastCGI');
		$options[] = SelectHelper::option('litespeed', 'LiteSpeed/lsapi');

		return self::genericlist($options, $name, $attribs, $selected, $name);
	}

	public static function minimumStability($name, $selected, $attribs = array())
	{
		$levels  = array('alpha', 'beta', 'rc', 'stable');
		$options = array();

		foreach ($levels as $level)
		{
			$options[] = SelectHelper::option($level, Language::_('COM_ADMINTOOLS_PARAMS_MINSTABILITY_' . strtoupper($level)));
		}

		return self::genericlist($options, $name, $attribs, $selected, $name);
	}

	public static function timezones($name, $attribs = array(), $selected)
	{
		if (empty($attribs))
		{
			$attribs = null;
		}
		else
		{
			$temp = '';

			foreach ($attribs as $key => $value)
			{
				$temp .= $key . ' = "' . $value . '"';
			}

			$attribs = $temp;
		}

		// We're going to use WordPress builtin function for this
		$html = '<select id="'.$name.'" name="'.$name.'" '.$attribs.'>';
		$html .= 	wp_timezone_choice( $selected, get_user_locale() );
		$html .= '</select>';

		return $html;
	}

	public static function roles($name, $selected, array $attribs = array())
	{
		$options = array();
		$roles   = get_editable_roles();

		foreach ($roles as $key => $role)
		{
			$options[] = SelectHelper::option($key, $role['name']);
		}

		if (!isset($attribs['hideEmpty']))
		{
			array_unshift($options, SelectHelper::option('', '- ' . Language::_('COM_ADMINTOOLS_LBL_COMMON_ROLES_SELECT') . ' -'));
		}
		else
		{
			unset($attribs['hideEmpty']);
		}

		return self::genericlist($options, $name, $attribs, $selected, $name);
	}

	public static function error_reporting($name, $selected, array $attribs = array())
	{
		$options[] = SelectHelper::option('', Language::_('COM_ADMINTOOLS_ERROR_REPORTING_DEFAULT'));
		$options[] = SelectHelper::option('none', Language::_('COM_ADMINTOOLS_ERROR_REPORTING_NONE'));
		$options[] = SelectHelper::option('errors', Language::_('COM_ADMINTOOLS_ERROR_REPORTING_ERRORS'));
		$options[] = SelectHelper::option('minimal', Language::_('COM_ADMINTOOLS_ERROR_REPORTING_MINIMAL'));
		$options[] = SelectHelper::option('full', Language::_('COM_ADMINTOOLS_ERROR_REPORTING_FULL'));
		$options[] = SelectHelper::option('developer', Language::_('COM_ADMINTOOLS_ERROR_REPORTING_DEVELOPER'));

		return self::genericlist($options, $name, $attribs, $selected, $name);
	}
}
